////// Fragment Shader
precision highp float;

uniform sampler2D inputImageTexture;
varying vec2 textureCoordinate;
uniform vec2 inputSize;
uniform highp float progress;
uniform highp float blurSize;
uniform highp float angle;
uniform int lowDevice;

#define H vec3(0.299, 0.587, 0.114)
float rgb2gray(vec3 rgb)
{
    float gray = dot(rgb, H);
    return gray;
}

vec4 edgeSoul(sampler2D inputImageTex, vec2 uv, vec2 bulrSize)
{
    vec4 n[9];
    float w = bulrSize.x;
    float h = bulrSize.y;
    n[0] = texture2D(inputImageTex, uv + vec2(-w, -h));
    n[1] = texture2D(inputImageTex, uv + vec2(0.0, -h));
    n[2] = texture2D(inputImageTex, uv + vec2(w, -h));
    n[3] = texture2D(inputImageTex, uv + vec2(-w, 0.0));
    n[4] = texture2D(inputImageTex, uv);
    n[5] = texture2D(inputImageTex, uv + vec2(w, 0.0));
    n[6] = texture2D(inputImageTex, uv + vec2(-w, h));
    n[7] = texture2D(inputImageTex, uv + vec2(0.0, h));
    n[8] = texture2D(inputImageTex, uv + vec2(w, h));
    float totalAlpha;
    for (int i = 0; i < 9; i += 1){
        totalAlpha += n[i].a;
    }
    vec4 edgeH = n[2] + (2.0*n[5]*1.0) + n[8] - (n[0] + (2.0*n[3]*1.0) + n[6]);
    vec4 edgeV = n[0] + (2.0*n[1]*1.0) + n[2] - (n[4] + (2.0*n[7]*1.0) + n[8]);
    vec4 sobel = sqrt((edgeH * edgeH) + (edgeV * edgeV));
    float alpha = totalAlpha / 9.0;
    float gray = rgb2gray(sobel.rgb*alpha);
    sobel = vec4(vec3(gray), alpha);
    return vec4(sobel.rgb, alpha);
}

vec2 getBlurSize(){
    highp float scale = inputSize.x / inputSize.y;
    highp float diff = 1.0 / 720.;
    if (scale < 1.0){
        highp float diff2 =diff*scale;
        return vec2(diff, diff*scale);
    } else {
        highp float diff2 =diff/scale;
        return vec2(diff2, diff);
    }
}

void main(){
    vec2 uv = textureCoordinate;
    vec4 base = texture2D(inputImageTexture, uv);
    vec4 baseColor = edgeSoul(inputImageTexture, uv, getBlurSize());
    vec4 resultColor = pow(baseColor, vec4(0.9983));
    resultColor.rgb = step(0.7874, 1.0-resultColor.rgb);
    gl_FragColor = resultColor*base.a;
}
