//////// Fragment Shader
precision highp float;
varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform highp float iTime;
uniform highp vec2 inputSize;
uniform float needBackWhite;

vec4 blackAndWhite(vec4 color)
{
    vec4 outputColor = vec4(dot(color.xyz, vec3(.299, .587, .114)));
    if (outputColor.a < 1.0) {
        outputColor.a = 1.0;
    }
    return outputColor;
}

void main() {
    vec2 uv;
    // 弧度比例
    float angle = .225;
    if (inputSize.x > inputSize.y) {
        uv = (textureCoordinate * inputSize.xy - inputSize.xy * 0.5) / inputSize.y;
        uv *= angle;
        uv.x *= inputSize.y / inputSize.x;
    } else {
        uv = (textureCoordinate * inputSize.xy - inputSize.xy * 0.5) / inputSize.x;
        uv *= angle;
        uv.y *= inputSize.x / inputSize.y;
    }
    // Fish eye
    float fovTheta = 7.2;
    float z = sqrt(0.2 - uv.x * uv.x - uv.y * uv.y);
    float a = 1. / (z * tan(fovTheta * 0.5));
    uv = uv * a;
    vec4 cola = texture2D(inputImageTexture, uv + 0.5);
    // Take video pixel
    vec3 col = cola.rgb;
    // Make borders
    col *= smoothstep(-0.51, -0.50, uv.x) * smoothstep(0.51, 0.50, uv.x);
    col *= smoothstep(-0.51, -0.50, uv.y) * smoothstep(0.51, 0.50, uv.y);

    if (needBackWhite > 0.0) {
        gl_FragColor = blackAndWhite(vec4(col, 1.));
        gl_FragColor = vec4(gl_FragColor.rgb, cola.a);
    }else{
        gl_FragColor = vec4(col, cola.a);
    }

}



