////// Fragment Shader
precision highp float;

uniform sampler2D inputImageTexture;
varying vec2 textureCoordinate;
uniform vec2 inputSize;
uniform highp float progress;
uniform highp float blurSize;
uniform highp float angle;
uniform int lowDevice;

vec4 edgeSoul(sampler2D inputImageTex, vec2 uv, vec2 bulrSize, float weight)
{
    vec4 n[9];
    float w = bulrSize.x;
    float h = bulrSize.y;
    n[0] = texture2D(inputImageTex, uv + vec2(-w, -h));
    n[1] = texture2D(inputImageTex, uv + vec2(0.0, -h));
    n[2] = texture2D(inputImageTex, uv + vec2(w, -h));
    n[3] = texture2D(inputImageTex, uv + vec2(-w, 0.0));
    n[4] = texture2D(inputImageTex, uv);
    n[5] = texture2D(inputImageTex, uv + vec2(w, 0.0));
    n[6] = texture2D(inputImageTex, uv + vec2(-w, h));
    n[7] = texture2D(inputImageTex, uv + vec2(0.0, h));
    n[8] = texture2D(inputImageTex, uv + vec2(w, h));
    float totalAlpha;
    for (int i=0;i<9;i+=1){
        totalAlpha += n[i].a;
    }
    vec4 edgeH = n[2] + (2.0*n[5]*weight) + n[8] - (n[0] + (2.0*n[3]*weight) + n[6]);
    vec4 edgeV = n[0] + (2.0*n[1]*weight) + n[2] - (n[6] + (2.0*n[7]*weight) + n[8]);
    vec4 sobel = sqrt((edgeH * edgeH) + (edgeV * edgeV));
    vec4 base = texture2D(inputImageTex, uv);
    float alpha = totalAlpha/9.0*base.a;
    return vec4(sobel.rgb, 1.0)*alpha;
}

vec2 getBlurSize(){
    highp float scale = inputSize.x / inputSize.y;
    highp float diff = 1.0 / 720.;
    if (scale < 1.0){
        highp float diff2 =diff*scale;
        return vec2(diff, diff*scale);
    } else {
        highp float diff2 =diff/scale;
        return vec2(diff2, diff);
    }
}

void main(){
    vec2 uv = textureCoordinate;
    //    gl_FragColor = texture2D(inputImageTexture, uv);
    gl_FragColor = edgeSoul(inputImageTexture, uv, getBlurSize(), 3.0);
}
