////// Fragment Shader
precision highp float;
varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform highp vec2 inputSize;
uniform highp float iTime;

#define CURVE //画面整体弯曲
#define SCANS //扫描线
#define FLICKS //RGB分离
#define GRAINS //颗粒
#define YBUG //上下抖动
#define DIRTY //污渍
#define STRIP //条纹扭曲
//#define COLOR //黑白
#define BLINK //闪烁
//#define VIG //边缘模糊

float rand2d(vec2 co) {
    return fract(sin(dot(co.xy, vec2(12.9898, 78.233))) * 43758.5453);
}

float rand(float n) {
    return fract(sin(n) * 43758.5453123);
}

float noise(float p) {
    float fl = floor(p);
    float fc = fract(p);
    return mix(rand(fl), rand(fl + 1.0), fc);
}

float map(float val, float amin, float amax, float bmin, float bmax) {
    float n = (val - amin) / (amax - amin);
    float m = bmin + n * (bmax - bmin);
    return m;
}

float snoise(float p) {
    return map(noise(p), 0.0, 1.0, -1.0, 1.0);
}

float threshold(float val, float cut) {
    float v = clamp(abs(val) - cut, 0.0, 1.0);
    v = sign(val) * v;
    float scale = 1.0 / (1.0 - cut);
    return v * scale;
}

float FREQUENCY = 1.0;

vec2 uv_curve(vec2 uv) {
    uv = (uv - 0.5) * 2.0;
    uv *= 1.1;
    uv.x *= 1.0 + pow((abs(uv.y) / 5.0), 2.0);
    uv.y *= 1.0 + pow((abs(uv.x) / 8.0), 2.0);
    uv /= 1.15;
    uv = (uv / 2.0) + 0.5;
    return uv;
}

vec3 changeColor(sampler2D tex, vec2 uv) {
    vec3 color = texture2D(inputImageTexture, uv).rgb;
    #ifdef COLOR
    float bw = (color.r + color.g + color.b) / 3.0;
    color = mix(color, vec3(bw, bw, bw), .95);
    float p = 1.5;
    color.r = pow(color.r, p);
    color.g = pow(color.g, p - 0.1);
    color.b = pow(color.b, p);
    #endif
    return color;
}

vec3 ghost(sampler2D tex, vec2 uv) {
    #ifdef FLICKS

    float speed = 5.0;
    float n1 = threshold(snoise(iTime * speed), .85);
    float n2 = threshold(snoise(2000.0 + iTime * speed), .85);
    float n3 = threshold(snoise(3000.0 + iTime * speed), .85);

    vec2 or = vec2(0., 0.);
    vec2 og = vec2(0, 0.);
    vec2 ob = vec2(0., 0);

    float os = .04;
    or += vec2(n1 * os, 0.);
    og += vec2(n2 * os, 0.);
    ob += vec2(0., n3 * os);

    float r = changeColor(inputImageTexture, uv + or).r;
    float g = changeColor(inputImageTexture, uv + og).g;
    float b = changeColor(inputImageTexture, uv + ob).b;
    vec3 color = vec3(r, g, b);
    return color;
    #else
    return texture2D(inputImageTexture, uv).rgb;
    #endif
}

vec2 uv_ybug(vec2 uv) {
    float n4 = clamp(noise(200.0 + iTime * 2.) * 14., 0., 2.);
    uv.y += n4;
    uv.y = mod(uv.y, 1.);
    return uv;
}

vec2 uv_hstrip(vec2 uv) {
    float vnoise = snoise(iTime * 6.);
    float hnoise = threshold(snoise(iTime * 10.), .5);

    float line = (sin(uv.y * 10. + vnoise) + 1.) / 2.;
    line = (clamp(line, .9, 1.) - .9) * 10.;

    uv.x += line * 0.03 * hnoise;
    uv.x = mod(uv.x, 1.);
    return uv;
}


void main()
{
    float t = float(int(iTime * FREQUENCY));
    vec2 uv = textureCoordinate;

    #ifdef CURVE
    uv = uv_curve(uv);
    #endif

    vec2 ouv = uv;

    #ifdef GRAINS
    float speed = 5.0;

    float xn = threshold(snoise(iTime * speed), .7) * 0.05;
    float yn = threshold(snoise((500.0 + iTime) * speed), .7) * 0.05;

    float r = rand2d(uv + (t + 100.0) * .01);
    uv = uv + vec2(xn, yn) * r;
    #endif

    #ifdef YBUG
    uv = uv_ybug(uv);
    #endif

    #ifdef STRIP
    uv = uv_hstrip(uv);
    #endif


    vec2 onePixel = vec2(0.0, 1.0) / inputSize.xy * 3.;
    #ifdef BLUR
    vec3 colorA = ghost(inputImageTexture, uv + onePixel, or, og, ob);
    vec3 colorB = ghost(inputImageTexture, uv - onePixel, or, og, ob);
    vec3 colorC = ghost(inputImageTexture, uv, or, og, ob);
    vec3 color = (colorA + colorB + colorC) / 3.0;
    #else
    vec3 color = ghost(inputImageTexture, uv);
    #endif

    //color = colorC;

    float scanA = (sin(uv.y * 3.1415 * inputSize.y / 2.7) + 1.) / 2.;
    float scanB = (sin(uv.y * 3.1415 * 1.) + 1.) / 2.;
    #ifdef SCANS
    color *= .75 + scanA * .25;
    //color *= .5 + scanC * .5;
    //color *= scanB;
    #endif

    #ifdef BLINK
    float blink = .96 + .04 * (sin(iTime * 100.) + 1.) / 2.;
    color *= blink;
    #endif

    #ifdef VIG
    float vig = 44.0 * (ouv.x * (1.0 - ouv.x) * ouv.y * (1.0 - ouv.y));
    vig *= mix(0.7, 1.0, rand(t + 0.5));
    color *= .6 + .4 * vig;
    #endif

    #ifdef DIRTY
    color *= 1.0 + rand2d(uv + t * .01) * 0.2;
    #endif

    vec3 backColor = vec3(.4, .4, .4);
    if (ouv.x < 0.0 || ouv.x > 1.0)
    color = backColor;
    if (ouv.y < 0.0 || ouv.y > 1.0)
    color = backColor;
    gl_FragColor = vec4(color, 1.0);
}